package org.eclipse.persistence.jpa.test.framework;

import java.io.File;
import java.lang.reflect.Field;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.eclipse.persistence.config.PersistenceUnitProperties;
import org.eclipse.persistence.internal.jpa.deployment.SEPersistenceUnitInfo;

public class EmfManager {
    private boolean _debug = true;
    private final Map<String, EntityManagerFactory> _emfs;

    public EmfManager() {
        _emfs = new HashMap<String, EntityManagerFactory>();
    }

    public void close() {
        for (EntityManagerFactory emf : _emfs.values()) {
            try {
                d("Closing [ " + emf + " ]");
                emf.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void inject(Object testInstance) {
        try {
            if (testInstance != null) {
                Class<?> cls = testInstance.getClass();
                // Keep track of injected EMFs for duplicates
                Set<EntityManagerFactory> injectedEmfs = new HashSet<EntityManagerFactory>();
                // Could be smarter and cache some of this
                for (Field field : cls.getDeclaredFields()) {
                    Emf emf = field.getAnnotation(Emf.class);
                    if (emf != null) {
                        field.setAccessible(true);
                        Object val = field.get(testInstance);
                        if (val == null) {
                            EntityManagerFactory factory = getEntityManagerFactory(emf);
                            if (!injectedEmfs.add(factory)) {
                                throw new RuntimeException(
                                    "Attempted to inject the same EntityManagerFactory multiple times into "
                                        + testInstance
                                        + ". Please remove the duplicate @Emf annotation, or change the name so that each "
                                        + "annotation is for a distinct EntityManagerFactory.");
                            }
                            field.setAccessible(true);
                            field.set(testInstance, factory);
                            d("Injected " + factory + " into " + testInstance);
                        }
                    }
                }
                cls.getAnnotation(Emf.class);

            }
        } catch (Exception e) {

        }
    }

    private EntityManagerFactory getEntityManagerFactory(Emf anno) {
        String name = anno.name();

        // Check cache
        EntityManagerFactory emf = _emfs.get(name);
        if (emf != null) {
            return emf;
        }

        List<String> mappingFiles = Arrays.asList(anno.mappingFiles());
        List<String> classes = new ArrayList<String>();
        Properties persistenceProperties = new Properties();

        for (Class<?> cls : anno.classes()) {
            classes.add(cls.getName());
        }

        for (Property prop : anno.properties()) {
            persistenceProperties.put(prop.name(), prop.value());
        }
        persistenceProperties.put(PersistenceUnitProperties.DDL_GENERATION, anno.createTables().toString());

        SEPersistenceUnitInfo pu = createSEPUInfo(anno.name(), classes, mappingFiles, persistenceProperties);

        Properties props = new Properties();
        props.put(PersistenceUnitProperties.ECLIPSELINK_SE_PUINFO, pu);

        emf = Persistence.createEntityManagerFactory(name, props);
        _emfs.put(name, emf);

        return emf;
    }

    private SEPersistenceUnitInfo createSEPUInfo(String puName, List<String> classes, List<String> mappingFiles,
        Properties persistenceProperties) {
        SEPersistenceUnitInfo pu = new SEPersistenceUnitInfo();
        pu.setClassLoader(Thread.currentThread().getContextClassLoader());
        pu.setPersistenceUnitName(puName);
        pu.setManagedClassNames(classes);
        pu.setMappingFileNames(mappingFiles);
        try {
            pu.setPersistenceUnitRootUrl(new File(".").toURI().toURL());
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        pu.setProperties(persistenceProperties);

        return pu;
    }

    private void d(String msg) {
        if (_debug) {
            System.err.println(msg);
        }
    }

}
